# <path to game directory>/addons/eventscripts/_engines/python/Lib/esutils/
# exceptions/__init__.py
# by Adam Cunnington

"""Catch exceptions raised and log them for reference / error tracing where 
stdout is unavailable to the user, i.e. on a hosted server.

"""

from __future__ import with_statement
import os
import sys
import time
import traceback

_ERROR_FILE = "errors.log"

__all__ = (
    "ExceptionsError", 
    "ExceptionManager", 
)

class ExceptionsError(Exception):
    """General error encountered relating to esutils.exceptions"""

class ExceptionManager(object):
    def __init__(self):
        self._log_errors = False
        self.sys_excepthook = sys.excepthook
        
    def _excepthook(self, *args, **kwargs):
        try:
            self._log_error(traceback.format_exception(*args, **kwargs))
        except:
            self.sys_excepthook(*sys.exc_info())
        finally:
            self.sys_excepthook(*args, **kwargs)
            
    def _get_log(self):
        return self._log_errors
        
    def _log_error(self, lines):
        with open(os.path.join(os.path.dirname(__file__), _ERROR_FILE), "a") \
             as fp:
            for line in lines:
                fp.write("%s\n" % line)
            fp.write("Error Encountered: %s\n\n\n" % time.strftime("%x (%X)"))
            
    def _set_log(self, value):
        if value:
            sys.excepthook = self._excepthook
        else:
            sys.excepthook = self.sys_excepthook
        self._log_errors = value
        
    log = property(_get_log, _set_log)